function [p,nodos] = astar (board, heuristicmethod)
% A* algorithm

% initialize the OPEN and CLOSED sets and their costs

start = initialState(board);

setOpen = [ start ]; 
setOpenCosts = [ start.cost ]; 
setOpenHeuristics = [ heuristic(heuristicmethod, board, start) ];

setClosed = []; 
setClosedCosts = [];

% keep track of the number of expanded nodes

nodos = 0;

% create figure so we can witness the magic
axishandle = createFigure(board,strcat('A* (',heuristicmethod,')'));

% Element in OPEN with smallest cost

[value, index] = min(setOpenCosts + setOpenHeuristics);
currentState = setOpen(index);

% as long as we have not found the goal or run out of spaces to explore

while (~finalState(board,currentState) & ~isempty(setOpen))
  
  nodos = nodos+1;

  % Put node in CLOSED
  
  setClosed = [setClosed; setOpen(index)];
  setClosedCosts = [setClosedCosts; currentState.cost];

  % Remove node form OPEN
  
  if (index > 1 & index < length(setOpen))
    setOpen = [setOpen(1:index-1); setOpen(index+1:end)];
    setOpenCosts = [setOpenCosts(1:index-1); setOpenCosts(index+1:end)];
    setOpenHeuristics = [setOpenHeuristics(1:index-1); setOpenHeuristics(index+1:end)];
  elseif (index == 1)
    setOpen = setOpen(2:end);
    setOpenCosts = setOpenCosts(2:end);
    setOpenHeuristics = setOpenHeuristics(2:end);
  else
    setOpen = setOpen(1:end-1);
    setOpenCosts = setOpenCosts(1:end-1);
    setOpenHeuristics = setOpenHeuristics(1:end-1);
  end

  % Following states

  action = actions(board,currentState);

  for jj=1:length(action)

    next(jj) = nextState(board,currentState,action(jj));
    next(jj).prev = length(setClosed);

    costs(jj) = next(jj).cost; % currentState.cost + board.cost(next(jj).x, next(jj).y); 
    heuristics(jj) = heuristic(heuristicmethod, board, next(jj));

    % if cost infinite, then it's a wall, so ignore
    if ~isinf(costs(jj))

      % if node is not in OPEN or CLOSED then insert into costchart and 
      % movement pointers, and put node in OPEN
      if ~included(next(jj),setClosed) & ~included(next(jj),setOpen)
        board.costchart(next(jj).x,next(jj).y) = costs(jj);
        
	setOpen = [setOpen; next(jj)];
        setOpenCosts = [setOpenCosts; costs(jj)];
        setOpenHeuristics = [setOpenHeuristics; heuristics(jj)];
      % else node has already been seen, so check to see if we have
      % found a better route to it.
      elseif included(next(jj),setOpen)
        I = included(next(jj),setOpen);
        % update if we have a better route
        if setOpenCosts(I) > costs(jj)
          board.costchart(next(jj).x,next(jj).y) = costs(jj);
          setOpenCosts(I) = costs(jj);
          setOpenHeuristics(I) = heuristics(jj);
	  setOpen(I).cost = costs(jj);
          setOpen(I).prev = length(setClosed);
        end
      % else node has already been CLOSED, so check to see if we have
      % found a better route to it.
      else
        % find relevant node in CLOSED
        I = included(next(jj), setClosed);
        % update if we have a better route
        if setClosedCosts(I) > costs(jj)
           disp('WARNING: Best route through closed node... Inadmissible heuristic?');
           board.costchart(next(jj).x,next(jj).y) = costs(jj);
           setClosedCosts(I) = costs(jj);
	   setClosed(I).cost = costs(jj);
           setClosed(I).prev = length(setClosed);
        end
      end
    end
  end
  
  if ~isempty(setOpen) 
     % element in OPEN with the smallest cost
     [value, index] = min(setOpenCosts + setOpenHeuristics);
     currentState = setOpen(index);

     % UI
     data = board.costchart';
     set(axishandle,'CData',[data data(:,end); data(end,:) data(end,end)]);
     % hack to make image look right
     set(gca,'CLim',[0 1.1*max(board.costchart(find(board.costchart < Inf)))]);
     drawnow; 
  end
  
end

if finalState(board,currentState)
  disp('Solucin encontrada :-)');
  disp(['- Heurstica: ', heuristicmethod])
  disp(['- Nodos expandidos: ', num2str(nodos)])
  % now find the way back using FIELDPOINTERS, starting from goal position
  p = path(board, currentState, setClosed);
  disp(['- Camino de longitud: ', num2str(length(p)-1)])
  disp(['- Camino de coste: ', num2str(currentState.cost)])
  % plot final path
  plot(p(:,2)+0.5,p(:,1)+0.5,'Color',0.2*ones(3,1),'LineWidth',4);
  drawnow;
  drawnow;
elseif isempty(setOpen)
  disp('No se encontr una solucin :-('); 
  disp(['- Heurstica: ', heuristicmethod])
  disp(['- Nodos expandidos: ', num2str(nodos)])
  p = [];
end



%%%

function ok = included (state, set)

ok = 0;

for i=1:length(set)
    if (sameState(state,set(i)))
       ok = i;
    end
end
