function axishandle = createFigure (board, title)
% This function creates a pretty figure of a given board

    % If there is no figure open, then create one
    if isempty(gcbf)
      fig = figure('name',title);
      set(fig,'Position',[50 50 500 500]);
      set(fig,'Units','Normalized');
      set(fig,'MenuBar','none');
      Caxes2 = axes('position', [0.01 0.01 0.98 0.98],'FontSize',12, ...
        'FontName','Helvetica');
    else
      % get the current figure, and clear it
      gcf; cla;
    end
    n = board.n;
    % plot board.field where walls are black, and everything else is white
    walls=board.cost';
    walls(walls < Inf) = 0;
    pcolor([1:n+1],[1:n+1],[walls walls(:,end); walls(end,:) walls(end,end)]);
    % set the colormap for the ploting the cost and looking really nice
    map = jet;
    cmap = flipud(map);
    % make first entry be white, and last be black
    cmap(1,:) = zeros(1,3); 
    cmap(end,:) = ones(1,3);
    % apply the colormap, but make red be closer to goal
    map = flipud(cmap);
    colormap(map);
    % keep the plot so we can plot over it
    hold on;
    % now plot the f values for all tiles evaluated
    data = board.costchart';
    axishandle = pcolor([1:n+1],[1:n+1],[data data(:,end); data(end,:) data(end,end)]);
    % plot goal as a red circle and start as a green circle

    for i=1:length(board.goal)
        plot(board.goal(i).x+0.5,board.goal(i).y+0.5,'go','MarkerSize',8,'LineWidth',4);
    end
    plot(board.start.x+0.5,board.start.y+0.5,'bo','MarkerSize',8,'LineWidth',4);
    % add a button so that can re-do the demonstration
    % uicontrol('Style','pushbutton','String','RE-DO', 'FontSize',12, 'Position', [1 1 60 40], 'Callback','astardemo');
